﻿using System;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using Buzzard.Xmpp.Stream.Elements;
using FakeItEasy;
using Microsoft.XmlDiffPatch;
using NUnit.Framework;

namespace Buzzard.Xmpp.Stream.Elements
{
    public abstract class XmppStreamElementTestBase
    {
        private XmlNamespace xmlNamespace;
        private string name;
        
        protected XmppStreamElementTestBase(string name, string namespaceUri, string namespaceName)
        {
            if (string.IsNullOrEmpty(name)) throw new ArgumentNullException("name");
            this.name = name;
            this.xmlNamespace = new XmlNamespace(namespaceName, namespaceUri);
        }

        protected string Name
        {
            get { return name; }
        }

        protected XmlNamespace XmlNamespace
        {
            get { return xmlNamespace; }
        }

        protected XElement CreateXElement(string name)
        {
            return CreateXElement(name, "", "");
        }

        protected XElement CreateXElement(string name, string namespaceUri)
        {
            return CreateXElement(name, "", namespaceUri);
        }

        protected XElement CreateXElement(string name, string namespaceName, string namespaceUri)
        {
            return XmlUtils.CreateElement(name, namespaceName, namespaceUri);
        }

        protected void VerifyWriteOutput(XElement expectedXml, XmppStreamElement element)
        {
            //XmlUtils.CreateXmlReader uses instead XElement.CreateReader
            //because last one don't treat 'xmlns' as attribute
            XmlReader expectedXmlReader = XmlUtils.CreateXmlReader(expectedXml);
            VerifyWriteOutput(expectedXmlReader, element);
        }

        protected void VerifyWriteOutput(string expectedXml, XmppStreamElement element)
        {
            XmlReader expectedXmlReader = XmlUtils.CreateXmlReader(expectedXml);
            VerifyWriteOutput(expectedXmlReader, element);
        }

        protected void VerifyWriteOutput(XmlReader expectedXml, XmppStreamElement element)
        {
            XmlReader acrtalXmlReader = CreateReaderForElement(element);
            XmlDiff diff = CreateXmlDiff();

            bool valid = diff.Compare(expectedXml, acrtalXmlReader);

            if (!valid) Assert.Fail("Expected {0} but was {1}", expectedXml, element);
        }

        private XmlReader CreateReaderForElement(XmppStreamElement element)
        {
            StringBuilder xml = new StringBuilder();
            XmlWriterSettings writerSettings = new XmlWriterSettings { OmitXmlDeclaration = true };
            XmlWriter writer = XmlWriter.Create(xml, writerSettings);
            
            element.Write(writer);
            writer.Close();

            return XmlReader.Create(new StringReader(xml.ToString()));
        }

        private XmlDiff CreateXmlDiff()
        {
            XmlDiff diff = new XmlDiff();
            diff.Algorithm = XmlDiffAlgorithm.Precise;
            diff.IgnoreWhitespace = true;
            diff.IgnoreChildOrder = true;
            return diff;
        }

        protected T CreateStreamElementMock<T>(string name, string namespaceUri = "", string namespaceName = "") where T : XmppStreamElement
        {
            Fake<T> fake = new Fake<T>(o => o.WithArgumentsForConstructor(new[] { name, namespaceName, namespaceUri }));
            fake.AnyCall().CallsBaseMethod();
            return fake.FakedObject;
        }
    }
}
