﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Buzzard.Xmpp.Stream.Elements;

namespace Buzzard.Xmpp
{
    public class RosterManager
    {
        private List<RosterItem> originalRoster; 
        private List<RosterItem> localRoster;

        public RosterManager()
        {
            localRoster = new List<RosterItem>();
            originalRoster = new List<RosterItem>();
        }

        public RosterManager(IEnumerable<RosterItem> initialRoster) : this()
        {
            ResetRoster(initialRoster);
        }

        public IEnumerable<RosterItem> AddedItems
        {
            get { return localRoster.Where(ri => !originalRoster.Contains(ri)); }
        }

        public IEnumerable<RosterItem> RemovedItems
        {
            get { return originalRoster.Where(ri => !localRoster.Contains(ri)); }
        }

        public ICollection<RosterItem> Roster
        {
            get { return localRoster; }
        }

        public void ResetRoster()
        {
            localRoster.Clear();
            originalRoster.Clear();
        }

        public void ResetRoster(IEnumerable<RosterItem> newRoster)
        {
            ResetRoster();
            localRoster.AddRange(newRoster);
            originalRoster.AddRange(newRoster);
        }

        public void SubmitChanges()
        {
            originalRoster.Clear();
            originalRoster.AddRange(localRoster);
        }

        public void RevertChanges()
        {
            localRoster.Clear();
            localRoster.AddRange(originalRoster);
        }

        public void PushItem(RosterItem item)
        {
            originalRoster.Add(item);
            localRoster.Add(item);
        }

        public void UpdateRoster(IqStanza rosterIq)
        {
            if (!rosterIq.IsRosterResult()) throw new InvalidOperationException();

            IEnumerable<RosterItem> items = rosterIq.GetRosterItemsElements().Select(ri => new RosterItem(ri.ItemName, ri.Jid));
            foreach (RosterItem item in items)
            {
                localRoster.Add(item);
            }
        }
    }
}
