﻿using System.Xml;

namespace Buzzard.Xmpp.Stream.Elements
{
    public abstract class DatagramElement : CompositeElement
    {
        protected DatagramElement(string name) : base(name)
        {
        }

        protected DatagramElement(string name, string namespaceUri) : base(name, namespaceUri)
        {
        }

        protected DatagramElement(string name, string namespaceName, string namespaceUri) : base(name, namespaceName, namespaceUri)
        {
        }

        protected DatagramElement(string name, object id)
            : this(name, null, null, id)
        { }

        protected DatagramElement(string name, JID from, JID to)
            : this(name, from, to, null)
        { }

        protected DatagramElement(string name, JID from, JID to, object id)
            : base(name)
        {
            From = from;
            To = to;
            Id = id;
            XmlLang = "";
        }

        public object Id 
        {
            get { return GetAttributeValue("id"); }
            set { SetAttributeValue("id", value); } 
        }

        public JID From
        {
            get { return (JID) GetAttributeValue("from"); }
            set { SetAttributeValue("from", value); }
        }

        public JID To
        {
            get { return (JID) GetAttributeValue("to"); }
            set { SetAttributeValue("to", value); }
        }

        public string XmlLang { get; set; }

        protected override void WriteAttributes(XmlWriter writer)
        {
            base.WriteAttributes(writer);
            if (!string.IsNullOrEmpty(XmlLang)) writer.WriteAttributeString("xml", "lang", null, XmlLang);
        }

        public override bool Equals(object obj)
        {
            DatagramElement element = obj as DatagramElement;
            return Equals(element);
        }

        public bool Equals(DatagramElement element)
        {
            return base.Equals(element) && AreValuesEquals(XmlLang, element.XmlLang);
        }
    }
}
